﻿local QuestLevelPatch = {}

--------------------------------------------------------------------------------------------------------
--                                    QuestLevelPatch variables                                       --
--------------------------------------------------------------------------------------------------------
local QuestLevel_original_GetQuestLogTitle = GetQuestLogTitle
local QuestLevel_original_GetGossipAvailableQuests = GetGossipAvailableQuests
local QuestLevel_original_GetGossipActiveQuests = GetGossipActiveQuests

--------------------------------------------------------------------------------------------------------
--                                QuestLevelPatch hooked funcitons                                    --
--------------------------------------------------------------------------------------------------------
function GetGossipAvailableQuests()
	-- name, level, isTrivial, isDaily, isRepeatable, isLegendary, ... = GetGossipAvailableQuests()
	local availableQuests = { QuestLevel_original_GetGossipAvailableQuests() }
	local i = 1
	while(availableQuests[i]) do
		availableQuests[i] = "["..availableQuests[i+1].."] "..availableQuests[i]
		i = i + 6
	end

	return QuestLevelPatch:unpackAvailableQuests(availableQuests)
end

function GetGossipActiveQuests()
	-- name, level, isTrivial, isDaily, isLegendary, ... = GetGossipActiveQuests()
	local activeQuests = { QuestLevel_original_GetGossipActiveQuests() }
	local i = 1
	while(activeQuests[i]) do
		activeQuests[i] = "["..activeQuests[i+1].."] "..activeQuests[i]
		i = i + 5
	end

	return QuestLevelPatch:unpackActiveQuests(activeQuests)
end

function GetQuestLogTitleA(questIndex)
	local title, level, suggestedGroup, isHeader, isCollapsed, isComplete, frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI, isTask, isStory = QuestLevel_original_GetQuestLogTitle(questIndex)
	if (title and (not isHeader)) then
		title = "["..level.."] "..title
	end

	return title, level, suggestedGroup, isHeader, isCollapsed, isComplete, frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI, isTask, isStory
end

-- Hook objective tracker
function SetBlockHeader_hook()
	for i = 1, GetNumQuestWatches() do
		local questID, title, questLogIndex, numObjectives, requiredMoney, isComplete, startEvent, isAutoComplete, failureTime, timeElapsed, questType, isTask, isStory, isOnMap, hasLocalPOI = GetQuestWatchInfo(i)
		if ( not questID ) then
			break
		end
		local oldBlock = QUEST_TRACKER_MODULE:GetExistingBlock(questID)
		if oldBlock then
			local oldBlockHeight = oldBlock.height
			local oldHeight = QUEST_TRACKER_MODULE:SetStringText(oldBlock.HeaderText, title, nil, OBJECTIVE_TRACKER_COLOR["Header"])
			local newTitle = "["..select(2, GetQuestLogTitle(questLogIndex)).."] "..title
			local newHeight = QUEST_TRACKER_MODULE:SetStringText(oldBlock.HeaderText, newTitle, nil, OBJECTIVE_TRACKER_COLOR["Header"])
			oldBlock:SetHeight(oldBlockHeight + newHeight - oldHeight);
		end
	end
end
hooksecurefunc(QUEST_TRACKER_MODULE, "Update", SetBlockHeader_hook)

--------------------------------------------------------------------------------------------------------
--                                    QuestLevelPatch funcitons                                       --
--------------------------------------------------------------------------------------------------------
function QuestLevelPatch:unpackAvailableQuests(t, i)
	i = i or 1
	if t[i] then
		return t[i], t[i+1], t[i+2], t[i+3], t[i+4], t[i+5], self:unpackAvailableQuests(t, i+6)
	end
end

function QuestLevelPatch:unpackActiveQuests(t, i)
	i = i or 1
	if t[i] then
		return t[i], t[i+1], t[i+2], t[i+3], t[i+4], self:unpackActiveQuests(t, i+5)
	end
end
